
.. DO NOT EDIT.
.. THIS FILE WAS AUTOMATICALLY GENERATED BY SPHINX-GALLERY.
.. TO MAKE CHANGES, EDIT THE SOURCE PYTHON FILE:
.. "_as_gen/math/run_paving_functions.py"
.. LINE NUMBERS ARE GIVEN BELOW.

.. only:: html

    .. note::
        :class: sphx-glr-download-link-note

        :ref:`Go to the end <sphx_glr_download__as_gen_math_run_paving_functions.py>`
        to download the full example code.

.. rst-class:: sphx-glr-example-title

.. _sphx_glr__as_gen_math_run_paving_functions.py:


Paving functions
================

This simple example shows how paving functions in Shamrock works

.. GENERATED FROM PYTHON SOURCE LINES 7-14

.. code-block:: Python
   :lineno-start: 9



    import matplotlib.pyplot as plt
    import numpy as np

    import shamrock








.. GENERATED FROM PYTHON SOURCE LINES 16-17

Set box size

.. GENERATED FROM PYTHON SOURCE LINES 17-21

.. code-block:: Python
   :lineno-start: 18


    box_size_x = 2.0
    box_size_y = 2.0








.. GENERATED FROM PYTHON SOURCE LINES 22-23

Particle set

.. GENERATED FROM PYTHON SOURCE LINES 23-32

.. code-block:: Python
   :lineno-start: 24


    parts = [
        {"x": 0.1 * box_size_x, "y": 0.1 * box_size_y, "color": "blue"},
        {"x": 0.7 * box_size_x, "y": 0.1 * box_size_y, "color": "red"},
        {"x": 0.1 * box_size_x, "y": 0.7 * box_size_y, "color": "green"},
        {"x": 0.7 * box_size_x, "y": 0.7 * box_size_y, "color": "black"},
    ]









.. GENERATED FROM PYTHON SOURCE LINES 33-34

Utility to plot the paving function

.. GENERATED FROM PYTHON SOURCE LINES 34-90

.. code-block:: Python
   :lineno-start: 36



    def add_rect(x, y, w, h):
        plt.gca().add_patch(
            plt.Rectangle((x, y), w, h, alpha=0.5, fill=True, facecolor="grey", linewidth=2)
        )


    def plot_paving_function(pav_func, pav_func_name, shear_x=0.0):

        plt.figure()
        for i in range(-2, 3):
            for j in range(-3, 4):
                # j = i
                add_rect(
                    0.05 + i * box_size_x + shear_x * j,
                    0.05 + j * box_size_y,
                    box_size_x - 0.1,
                    box_size_y - 0.1,
                )

                for part in parts:
                    x, y, z = pav_func.f((part["x"], part["y"], 0.0), i, j, 0)

                    plt.scatter(x, y, color=part["color"])
        plt.title(f"Paving function: {pav_func_name}")
        plt.xlabel("x")
        plt.ylabel("y")

        plt.figure()
        for i in range(-2, 3):
            for j in range(-3, 4):
                add_rect(
                    0.05 + i * box_size_x + shear_x * j,
                    0.05 + j * box_size_y,
                    box_size_x - 0.1,
                    box_size_y - 0.1,
                )

                for part in parts:
                    x, y, z = pav_func.f((part["x"], part["y"], 0.0), i, j, 0)

                    x_2, y_2, z_2 = pav_func.f_inv((x, y, 0.0), i, j, 0)

                    delta_x = x_2 - part["x"]
                    delta_y = y_2 - part["y"]

                    if abs(delta_x) > 1e-4 or abs(delta_y) > 1e-4:
                        print("error")

                    plt.scatter(x_2 + 0.1 * i, y_2 + 0.1 * j, color=part["color"])
        plt.title(f"Paving function inverse: {pav_func_name}")
        plt.xlabel("x")
        plt.ylabel("y")









.. GENERATED FROM PYTHON SOURCE LINES 91-93

Testing the paving functions
----------------------------

.. GENERATED FROM PYTHON SOURCE LINES 95-96

Periodic paving function

.. GENERATED FROM PYTHON SOURCE LINES 96-105

.. code-block:: Python
   :lineno-start: 97


    plot_paving_function(
        shamrock.math.paving_function_general_3d(
            (box_size_x, box_size_y, 0.0), (box_size_x / 2.0, box_size_y / 2.0, 0.0), True, True, True
        ),
        "Periodic box",
    )





.. rst-class:: sphx-glr-horizontal


    *

      .. image-sg:: /_as_gen/math/images/sphx_glr_run_paving_functions_001.png
         :alt: Paving function: Periodic box
         :srcset: /_as_gen/math/images/sphx_glr_run_paving_functions_001.png
         :class: sphx-glr-single-img

    *

      .. image-sg:: /_as_gen/math/images/sphx_glr_run_paving_functions_002.png
         :alt: Paving function inverse: Periodic box
         :srcset: /_as_gen/math/images/sphx_glr_run_paving_functions_002.png
         :class: sphx-glr-single-img





.. GENERATED FROM PYTHON SOURCE LINES 106-107

Periodic & reflective paving function

.. GENERATED FROM PYTHON SOURCE LINES 107-116

.. code-block:: Python
   :lineno-start: 108


    plot_paving_function(
        shamrock.math.paving_function_general_3d(
            (box_size_x, box_size_y, 0.0), (box_size_x / 2.0, box_size_y / 2.0, 0.0), False, True, True
        ),
        "reflective in x periodic in y",
    )





.. rst-class:: sphx-glr-horizontal


    *

      .. image-sg:: /_as_gen/math/images/sphx_glr_run_paving_functions_003.png
         :alt: Paving function: reflective in x periodic in y
         :srcset: /_as_gen/math/images/sphx_glr_run_paving_functions_003.png
         :class: sphx-glr-single-img

    *

      .. image-sg:: /_as_gen/math/images/sphx_glr_run_paving_functions_004.png
         :alt: Paving function inverse: reflective in x periodic in y
         :srcset: /_as_gen/math/images/sphx_glr_run_paving_functions_004.png
         :class: sphx-glr-single-img





.. GENERATED FROM PYTHON SOURCE LINES 117-118

Fully reflective paving function

.. GENERATED FROM PYTHON SOURCE LINES 118-127

.. code-block:: Python
   :lineno-start: 119


    plot_paving_function(
        shamrock.math.paving_function_general_3d(
            (box_size_x, box_size_y, 0.0), (box_size_x / 2.0, box_size_y / 2.0, 0.0), False, False, True
        ),
        "Fully reflective",
    )





.. rst-class:: sphx-glr-horizontal


    *

      .. image-sg:: /_as_gen/math/images/sphx_glr_run_paving_functions_005.png
         :alt: Paving function: Fully reflective
         :srcset: /_as_gen/math/images/sphx_glr_run_paving_functions_005.png
         :class: sphx-glr-single-img

    *

      .. image-sg:: /_as_gen/math/images/sphx_glr_run_paving_functions_006.png
         :alt: Paving function inverse: Fully reflective
         :srcset: /_as_gen/math/images/sphx_glr_run_paving_functions_006.png
         :class: sphx-glr-single-img





.. GENERATED FROM PYTHON SOURCE LINES 128-129

Periodic & reflective paving function with shear

.. GENERATED FROM PYTHON SOURCE LINES 129-144

.. code-block:: Python
   :lineno-start: 130


    plot_paving_function(
        shamrock.math.paving_function_general_3d_shear_x(
            (box_size_x, box_size_y, 0.0),
            (box_size_x / 2.0, box_size_y / 2.0, 0.0),
            False,
            True,
            True,
            0.3,
        ),
        "reflective in x periodic in y with shear",
        shear_x=0.3,
    )

    plt.show()



.. rst-class:: sphx-glr-horizontal


    *

      .. image-sg:: /_as_gen/math/images/sphx_glr_run_paving_functions_007.png
         :alt: Paving function: reflective in x periodic in y with shear
         :srcset: /_as_gen/math/images/sphx_glr_run_paving_functions_007.png
         :class: sphx-glr-single-img

    *

      .. image-sg:: /_as_gen/math/images/sphx_glr_run_paving_functions_008.png
         :alt: Paving function inverse: reflective in x periodic in y with shear
         :srcset: /_as_gen/math/images/sphx_glr_run_paving_functions_008.png
         :class: sphx-glr-single-img






.. rst-class:: sphx-glr-timing

   **Total running time of the script:** (0 minutes 4.900 seconds)

**Estimated memory usage:**  71 MB


.. _sphx_glr_download__as_gen_math_run_paving_functions.py:

.. only:: html

  .. container:: sphx-glr-footer sphx-glr-footer-example

    .. container:: sphx-glr-download sphx-glr-download-jupyter

      :download:`Download Jupyter notebook: run_paving_functions.ipynb <run_paving_functions.ipynb>`

    .. container:: sphx-glr-download sphx-glr-download-python

      :download:`Download Python source code: run_paving_functions.py <run_paving_functions.py>`

    .. container:: sphx-glr-download sphx-glr-download-zip

      :download:`Download zipped: run_paving_functions.zip <run_paving_functions.zip>`


.. only:: html

 .. rst-class:: sphx-glr-signature

    `Gallery generated by Sphinx-Gallery <https://sphinx-gallery.github.io>`_
